/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include <sstream>
#include <iostream>

using namespace std;
using namespace FD;

class CvfdhSaveImage;
   
DECLARE_NODE(CvfdhSaveImage)
/*Node
 *
 * @name CvfdhSaveImage
 * @category FDOPENCV_HARK:ImageIO
 * @description View an image
 *
 * @input_name IMAGEIN
 * @input_description Image to visualize
 * @input_type CvfdImage
 *
 * @input_name OPERATION_FLAG
 * @input_type any
 * @input_description Integer or bool flag. The data saving is done if this flag is 1/true. Set this 0/false and make this 1/true at appropriate time since this block costs a lot of computation power. If this flag is 0/false, the operation is done only when this is 1/true. 
 *
 * @parameter_name FILENAME_PREFIX
 * @parameter_type string
 * @parameter_value img
 * @parameter_description Prefix of the saved file name
 *
 * @parameter_name INTERVAL
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Interval of file storing.
 *
 * @parameter_name FILE_FORMAT
 * @parameter_type string
 * @parameter_value jpg
 * @parameter_list jpg:bmp:png:ppm:tiff:exr:jp2
 * @parameter_description Selection of the image format. jpg = JPEG files, bmp = Windows bitmaps, png = Portable Network Graphics, ppm = Portable image format, tiff = TIFF files, exr = OpenEXR HDR images, jp2 = JPEG 2000 images 
 *
 * @output_name IMAGEOUT
 * @output_description Same as input image 
 * @output_type CvfdImage
 *
 * @parameter_name DEBUG
 * @parameter_type bool
 * @parameter_value false
 * @parameter_description Debug print.
 *
 END*/
   
   
class CvfdhSaveImage : public BufferedNode {
      
  int m_imageInID;
  int operationID;
  int m_imageOutID;
  string filename_prefix;
  int interval;
  bool isDebug;
  string file_format;
  int operation_flag;
            
public:
  CvfdhSaveImage(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    operationID = addInput("OPERATION_FLAG");
    m_imageOutID = addOutput("IMAGEOUT");         
    filename_prefix = object_cast<String>(parameters.get("FILENAME_PREFIX"));
    interval = dereference_cast<int>(parameters.get("INTERVAL"));
    isDebug = dereference_cast<bool>(parameters.get("DEBUG"));

    file_format = object_cast<String>(parameters.get("FILE_FORMAT"));
    operation_flag = 0;
    inOrder = true;
  }
      
  ~CvfdhSaveImage(){}
      
  void calculate(int output_id, int count, Buffer &out)
  {

    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count); 
    (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);

    // do not save if the image is NULL
    if(imagePtr->getImage() == NULL){
      return;
    }

    // do not save if interval is not coming yet.
    if(count % interval != 0){
      (*(outputs[m_imageOutID].buffer))[count] = NULL;
      return;
    }

    ObjectRef operation = getInput(operationID, count);
    if (typeid(*operation) == typeid(Bool)) { 
      bool bInput = dereference_cast<bool>(operation); 
      operation_flag = (bInput ? 1 : 0); 
    }else if(typeid(*operation) == typeid(Int)){ 
      operation_flag = dereference_cast<int> (operation); 
    }else{ 
      throw new NodeException(this, string("Input type is not appropriate in CMSave."), __FILE__, __LINE__); 
    } 

    if (operation_flag != 1) return;
    
    ostringstream os;
    os << filename_prefix << count << "." << file_format;

    if(isDebug){
      cout << os.str() << endl;
    }
         
    cvSaveImage (os.str().c_str(), imagePtr->getImage());

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhSaveImage)
};
