/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include <map>
#include <string>

using namespace std;
using namespace FD;

class CvfdThreshold;
   
DECLARE_NODE(CvfdThreshold)
/*Node
 *
 * @name CvfdThreshold
 * @category FDOPENCV_ORG:Morphological Operations
 * @description Applies fixed-level threshold to array elements
 *
 * @input_name IMAGEIN
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @output_name IMAGEOUT
 * @output_description Output image
 * @output_type CvfdImage
 *
 * @parameter_name THRESHOLD
 * @parameter_type float
 * @parameter_value 50
 * @parameter_description Threshold value
 *
 * @parameter_name MAX_VALUE
 * @parameter_type float
 * @parameter_value 255
 * @parameter_description Maximum value to use with CV_THRESH_BINARY and CV_THRESH_BINARY_INV thresholding types
 *
 * @parameter_name THRESHOLD_TYPE
 * @parameter_type string
 * @parameter_value CV_THRESH_BINARY;CV_THRESH_BINARY_INV;CV_THRESH_TRUNC;CV_THRESH_TOZERO;CV_THRESH_TOZERO_INV
 * @parameter_description Thresholding type
 *
 END*/
   
   
class CvfdThreshold : public BufferedNode {
  //Input ID
  int m_imageInID;
  //Output ID
  int m_imageOutID;
  //Parameters
  float m_threshold;
  float m_max_value;
  string m_thresholdType;
  //Initialize the thresholeType map
  map<string,int> m_thresholeTypeMap;
      
      
public:
  CvfdThreshold(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
         
    m_imageInID = addInput("IMAGEIN");
    m_imageOutID = addOutput("IMAGEOUT");
    m_threshold = dereference_cast<float>(parameters.get("THRESHOLD"));
    m_max_value = dereference_cast<float>(parameters.get("MAX_VALUE"));
    m_thresholdType = object_cast<String>(parameters.get("THRESHOLD_TYPE"));
         
    //initialize type map
    m_thresholeTypeMap["CV_THRESH_BINARY"] = CV_THRESH_BINARY;
    m_thresholeTypeMap["CV_THRESH_BINARY_INV"] = CV_THRESH_BINARY_INV;
    m_thresholeTypeMap["CV_THRESH_TRUNC"] = CV_THRESH_TRUNC;
    m_thresholeTypeMap["CV_THRESH_TOZERO"] = CV_THRESH_TOZERO;
    m_thresholeTypeMap["CV_THRESH_TOZERO_INV"] = CV_THRESH_TOZERO_INV;
         
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
    if(imagePtr->getImage() == NULL)
      {
	out[count] = ObjectRef(imagePtr);
	return;
      }
    CvfdImage* imageGRAY = imagePtr->gray(); 
    CvfdImage* image =  new CvfdImage(cvSize(imagePtr->getImage()->width, imagePtr->getImage()->height)
				      , IPL_DEPTH_8U, 1);
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__; 
    OPENCV_CALL(cvThreshold(imageGRAY->getImage(), image->getImage(), m_threshold, m_max_value, m_thresholeTypeMap[m_thresholdType]));         
    __END__;
    cvSetErrMode( status );
    delete imageGRAY;
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to threshold the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
         
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdThreshold)
};
